@@ -1,26 +1,25 @@
-{-# LANGUAGE FlexibleInstances, DeriveDataTypeable #-}
--- Full-featured argument parsing library for Haskell programs
--- Bart Massey <bart@cs.pdx.edu>
+{-# LANGUAGE FlexibleInstances, DeriveDataTypeable, Safe, CPP #-}
+------------------------------------------------------------
+-- |
+-- Module      :  System.Console.ParseArgs
+-- Description :  Full-featured command-line argument parsing library.
+-- Copyright   :  (c) 2007 Bart Massey
+-- License     :  BSD-style (see the file COPYING)
+-- Maintainer  :  Bart Massey <bart.massey@gmail.com>
+-- Stability   :  stable
+-- Portability :  portable
+--
+-- `ParseArgs` is a full-featured command-line argument
+-- parsing library.
+--
+-- This module supplies an argument parser.  Given a
+-- description of type [`Arg`] of the legal arguments to the
+-- program, a list of argument strings, and a bit of extra
+-- information, the `parseArgs` function in this module
+-- returns an `Args` data structure suitable for querying
+-- using the provided functions `gotArg`, `getArg`, etc.
+------------------------------------------------------------
 
--- Copyright © 2007-2010 Bart Massey
--- ALL RIGHTS RESERVED
-
--- You can redistribute and/or modify this library under the
--- terms of the "3-clause BSD LICENSE", as stated in the file
--- COPYING in the top-level directory of this distribution.
--- 
--- This library is distributed in the hope that it will be
--- useful, but WITHOUT ANY WARRANTY; without even the
--- implied warranty of MERCHANTABILITY or FITNESS FOR A
--- PARTICULAR PURPOSE.
-
--- |This module supplies an argument parser.
--- Given a description of type [`Arg`] of the legal
--- arguments to the program, a list of argument strings,
--- and a bit of extra information, the `parseArgs` function
--- in this module returns an
--- `Args` data structure suitable for querying using the
--- provided functions `gotArg`, `getArg`, etc.
 module System.Console.ParseArgs (
   -- * Describing allowed arguments
   -- |The argument parser requires a description of
@@ -31,8 +30,8 @@ module System.Console.ParseArgs (
   Argtype(..), 
   ArgsComplete(..),
   ArgsDash(..),
-  ArgsParseControl(..),
   APCData(..),
+  ArgsParseControl(..),
   -- ** DataArg and its pseudo-constructors
   DataArg,
   argDataRequired, argDataOptional, argDataDefaulted,
@@ -46,7 +45,7 @@ module System.Console.ParseArgs (
   -- |The argument parser returns an opaque map
   -- from argument index to parsed argument data
   -- (plus some convenience information).
-  ArgRecord, Args(..),
+  Args(..),
   parseArgs, parseArgsIO,
   -- ** Using parse results
   -- |Query functions permit checking for the existence
@@ -64,7 +63,11 @@ where
 
 import Control.Exception
 import Control.Monad
+#if __GLASGOW_HASKELL__ < 710
+import Control.Monad.ST.Safe
+#else
 import Control.Monad.ST
+#endif
 import Data.List
 import qualified Data.Map as Map
 import Data.Maybe
@@ -172,10 +175,12 @@ data Argval = ArgvalFlag   -- ^For simple present vs not-present flags.
 -- |The type of the mapping from argument index to value.
 newtype ArgRecord a = ArgRecord (Map.Map a Argval)
 
--- |The data structure `parseArgs` produces.  The key
--- element is the `ArgRecord` `args`.
+-- |The data structure `parseArgs` produces. There is a should-be-hidden
+-- field that describes the parse.
 data (Ord a) => Args a =
-    Args { args :: ArgRecord a      -- ^The argument map.
+    Args { __args :: ArgRecord a    -- ^The argument parse, only listed here
+                                    -- to work around a Haddock bug. See
+                                    -- <https://github.com/haskell/haddock/issues/456>.
          , argsProgName :: String   -- ^Basename of 0th argument.
          , argsUsage :: String      -- ^Full usage string.
          , argsRest :: [ String ]   -- ^Remaining unprocessed arguments.
@@ -336,13 +341,16 @@ data ArgsDash = ArgsHardDash   -- ^If an argument begins with
 
 -- |Record containing the collective parse control information.
 data ArgsParseControl = ArgsParseControl {
+  -- |Level of completeness of parse.
   apcComplete :: ArgsComplete,
+  -- |Handling of dashes in parse.
   apcDash :: ArgsDash }
 
 -- |Class for building parse control information,
 -- for backward compatibility.
 class APCData a where
-  getAPCData :: a -> ArgsParseControl
+  getAPCData :: a -> ArgsParseControl  -- ^Build an 'ArgsParseControl'
+                                       -- structure from the given info.
 
 instance APCData ArgsParseControl where
   getAPCData a = a
@@ -398,7 +406,7 @@ parseArgs apcData argd pathname argv =
            unless (and (map (check_present usage am) required_args))
                   (error "internal error")
            let am' = foldl supply_defaults am argd
-           return (Args { args = ArgRecord am',
+           return (Args { __args = ArgRecord am',
                           argsProgName = prog_name,
                           argsUsage = usage,
                           argsRest = rest }))
@@ -567,23 +575,26 @@ gotArg :: (Ord a) =>
           Args a    -- ^Parsed arguments.
        -> a         -- ^Index of argument to be checked for.
        -> Bool      -- ^True if the arg was present.
-gotArg (Args { args = ArgRecord am }) k =
+gotArg (Args { __args = ArgRecord am }) k =
     case Map.lookup k am of
       Just _ -> True
       Nothing -> False
 
 -- |Type of values that can be parsed by the argument parser.
 class ArgType b where
+
     -- |Fetch an argument's value if it is present.
     getArg :: (Show a, Ord a)
            => Args a    -- ^Parsed arguments.
            -> a         -- ^Index of argument to be retrieved.
            -> Maybe b   -- ^Argument value if present.
+
     -- |Fetch the value of a required argument.
     getRequiredArg :: (Show a, Ord a)
            => Args a    -- ^Parsed arguments.
            -> a         -- ^Index of argument to be retrieved.
            -> b   -- ^Argument value.
+
     getRequiredArg ads index =
         case getArg ads index of
           Just v -> v
@@ -591,7 +602,7 @@ class ArgType b where
                           ++ show index ++ "not supplied")
 
 getArgPrimitive :: Ord a => (Argval -> Maybe b) -> Args a -> a -> Maybe b
-getArgPrimitive decons (Args { args = ArgRecord am }) k =
+getArgPrimitive decons (Args { __args = ArgRecord am }) k =
   Map.lookup k am >>= decons
 
 instance ArgType () where
